import abc
import csv
import os
import sqlite3
import sys


class Pet:
    def __init__(self, name, age, type):
        self.name = name
        self.age = age
        self.type = type

    def __repr__(self):
        return '{}, a {} year old {}'.format(self.name, self.age, self.type)


class DataSource(abc.ABC):
    @abc.abstractmethod
    def get_pets_by_owner(self, owner_name):
        pass

    @abc.abstractmethod
    def get_pets_by_type(self, type_name):
        pass

    @abc.abstractmethod
    def get_all_pets(self):
        pass


class CsvSource(DataSource):
    def __init__(self, filename):
        self._pets_by_owner = {}
        self._pets_by_type = {}
        self._all_pets = []

        with open(filename) as f:
            reader = csv.DictReader(f)

            for row in reader:
                owner = row['Owner']
                name = row['Name']
                age = int(row['Age'])
                # type is a built-in Python function, so we dont want to use
                # it as a variable name
                pet_type = row['Type']

                pet = Pet(name, age, pet_type)

                self._all_pets.append(pet)

                if owner not in self._pets_by_owner:
                    self._pets_by_owner[owner] = []

                self._pets_by_owner[owner].append(pet)

                if pet_type not in self._pets_by_type:
                    self._pets_by_type[pet_type] = []

                self._pets_by_type[pet_type].append(pet)

        print(self._pets_by_owner)
        print(self._pets_by_type)
        print(self._all_pets)

    def get_all_pets(self):
        return self._all_pets

    def get_pets_by_owner(self, owner_name):
        if owner_name in self._pets_by_owner:
            return self._pets_by_owner[owner_name]
        else:
            return []

    def get_pets_by_type(self, type_name):
        if type_name in self._pets_by_type:
            return self._pets_by_type[type_name]
        else:
            return []


class DatabaseSource(DataSource):
    def __init__(self, filename):
        if not os.path.isfile(filename):
            sys.exit('{} does not exist, please run build_db.py first')

        self._conn = sqlite3.connect(filename)
        self._conn.row_factory = sqlite3.Row

    def get_all_pets(self):
        cur = self._conn.cursor()

        cur.execute('SELECT pet.name as petname, pet.age, '
                    '       pet_type.name as typename '
                    'FROM pet, pet_type '
                    'WHERE pet.type_id = pet_type.type_id')

        pets = []

        for row in cur.fetchall():
            pets.append(Pet(row['petname'], row['age'], row['typename']))

        return pets

    def get_pets_by_owner(self, owner_name):
        cur = self._conn.cursor()

        cur.execute('SELECT pet.name as petname, pet.age, '
                    '       pet_type.name as typename '
                    'FROM pet, pet_type, owner '
                    'WHERE pet.type_id = pet_type.type_id '
                    'AND pet.owner_id = owner.owner_id '
                    'AND owner.name = ?', (owner_name,))

        pets = []

        for row in cur.fetchall():
            pets.append(Pet(row['petname'], row['age'], row['typename']))

        return pets

    def get_pets_by_type(self, type_name):
        cur = self._conn.cursor()

        cur.execute('SELECT pet.name as petname, pet.age, '
                    '       pet_type.name as typename '
                    'FROM pet, pet_type '
                    'WHERE pet.type_id = pet_type.type_id '
                    'AND pet_type.name = ?', (type_name,))

        pets = []

        for row in cur.fetchall():
            pets.append(Pet(row['petname'], row['age'], row['typename']))

        return pets
